"""
Utility module for functions commonly used by store scripts
"""
from decimal import Decimal


def clean_price_string(price_string):
    """
    Removes most common formatting of a string that represents a price
    leaving it only with its numbers.
    """

    blacklist = ['CLP$', '$', '.', ',', '&nbsp;', '\r', '\n', '\t']

    for item in blacklist:
        price_string = price_string.replace(item, '')

    return price_string


def count_decimals(value):
    """
    Returns the number of decimals in a Decimal value
    e.g.; count_decimals(Decimal("1234.56")) = 2
    """
    return abs(value.as_tuple().exponent)


def format_currency(value, curr='$', sep='.', dp=',',
             pos='', neg='-', trailneg=''):
    """Convert Decimal to a money formatted string.

    curr: optional currency symbol before the sign (may be blank)
    sep: optional grouping separator (comma, period, space, or blank)
    dp: decimal point indicator (comma or period)
    only specify as blank when places is zero
    pos: optional sign for positive numbers: '+', space or blank
    neg: optional sign for negative numbers: '-', '(', space or blank
    trailneg:optional trailing minus indicator: '-', ')', space or blank

    """

    places = count_decimals(value)

    if not places:
        dp = ''

    q = Decimal(10) ** -places  # 2 places --> '0.01'
    sign, digits, exp = value.quantize(q).as_tuple()
    result = []
    digits = map(str, digits)
    build, next = result.append, digits.pop
    if sign:
        build(trailneg)
    for i in range(places):
        build(next() if digits else '0')
    build(dp)
    if not digits:
        build('0')
    i = 0
    while digits:
        build(next())
        i += 1
        if i == 3 and digits:
            i = 0
            build(sep)
    build(curr)
    build(neg if sign else pos)
    return ''.join(reversed(result))



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Ripley(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        soup = BeautifulSoup(browser.open(url).get_data())

        try:
            name = soup.find('span', {'class': 'textogrisbold'})
            name = name.string.encode('ascii', 'ignore')
        except AttributeError:
            return None

        prices = []

        try:
            price = soup.find('span', {'class': 'normalHOME'}).string
            price = Decimal(clean_price_string(price))
            prices.append(price)
        except Exception:
            pass

        try:
            price = soup.find('div', {'class': 'textodetallesrojo'})
            price = price.find('div').string.split('$')[1]
            price = Decimal(clean_price_string(price))
            prices.append(price)
        except Exception:
            pass

        try:
            price = soup.find('div', {'class': 'textodetallesrojo'}).string
            price = price.split('$')[1]
            price = Decimal(clean_price_string(price))
            prices.append(price)
        except Exception:
            pass

        if not prices:
            raise Exception

        price = min(prices)

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.ripley.cl/webapp/wcs/stores/servlet/'

        category_urls = [
            ['categoria-TVRipley-10051-001772-130000-ESP-N--', 'Notebook'],
            ['categoria-TVRipley-10051-001860-130000-ESP-N--', 'Notebook'],
            ['categoria-TVRipley-10051-013040-230000-ESP-N', 'Television'],
        ]

        browser = mechanize.Browser()

        product_links = {}

        for category_url, ptype in category_urls:
            if ptype not in product_types:
                continue
            j = 1
            while True:
                url = url_base + category_url + '?curPg=' + str(j)

                soup = BeautifulSoup(browser.open(url).get_data())

                p_paragraphs = soup.findAll('td', {'class': 'grisCatalogo'})
                p_paragraphs = p_paragraphs[1::3]
                p_paragraphs = [pp.parent.parent for pp in p_paragraphs]

                for p in p_paragraphs:
                    url = url_base + p.find('a')['href']
                    url = url.encode('ascii', 'ignore')
                    if url not in product_links:
                        product_links[url] = ptype

                next_page_link = soup.findAll('a',
                        {'class': 'linknormal4'})[-1]
                if next_page_link.string.strip() != '&gt;&gt;':
                    break

                j += 1

        return product_links.items()



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class PortatilChile(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        product_data = browser.open(url).get_data()
        product_soup = BeautifulSoup(product_data)

        name = product_soup.find('th', {'colspan': '2'}).string
        name = name.encode('ascii', 'ignore').split('"')[1]

        price = product_soup.find('table', {'cellspacing': '1'}).find('table')
        price = price.findAll('td')[3].find('strong').string
        price = Decimal(clean_price_string(price))

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        if 'Notebook' not in product_types:
            return []

        urlBase = 'http://www.portatilchile.cl/productos.html'

        browser = mechanize.Browser()
        soup = BeautifulSoup(browser.open(urlBase).get_data())

        product_tables = soup.findAll('table', {'width': '226'})

        product_links = []

        for product_table in product_tables:
            link = product_table.find('a')['href']
            product_links.append([link, 'Notebook'])

        return product_links



import unittest
from storescrapper.store import Store


class TestStoreClassShouldRaiseExceptionWhenUsedWithoutBeingSubclassed(
    unittest.TestCase):

    def setUp(self):
        self.store = Store()

    def test_product_urls_and_types_with_async(self):
        with self.assertRaises(NotImplementedError):
            self.store.product_urls_and_types(product_types=[u'Notebook', ])

    def test_product_urls_and_types_without_async(self):
        with self.assertRaises(NotImplementedError):
            self.store.product_urls_and_types(product_types=[u'Processor', ],
                async=False)

    def test_retrieve_product(self):
        with self.assertRaises(NotImplementedError):
            self.store.retrieve_product(
                url=u'http://www.example.com/dummy_product',
                product_type=u'PowerSupply')

    def test_products_with_async(self):
        with self.assertRaises(NotImplementedError):
            self.store.products(product_types=[u'Notebook', ])

    def test_products_without_async(self):
        with self.assertRaises(NotImplementedError):
            self.store.products(product_types=[u'Notebook', ], async=False)



from decimal import Decimal
import json
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Entel(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        try:
            product_data = browser.open(url).get_data()
        except mechanize.HTTPError:
            return None

        soup = BeautifulSoup(product_data)

        product_title = soup.find('h1').contents[0].strip()
        product_title = product_title.encode('ascii', 'ignore')

        product_prices = []

        pricing_options_titles = soup.findAll('h3')
        for title in pricing_options_titles:
            if title.string in ['Pack Prepago', 'Compra equipo']:
                price_container = title.parent.find('span',
                        {'class': 'valorPrecio'})
                if price_container:
                    price = clean_price_string(price_container.string)
                    product_prices.append(Decimal(price))

        if product_prices:
            product_price = min(product_prices)
        else:
            product_price = 0

        return [product_title, product_price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        if 'Cell' not in product_types:
            return []

        url = 'http://personas.entelpcs.cl/PortalPersonas/com/entelpcs/' \
              'catalogoEquipos/publico/controllers/listadoEquipos/' \
              'ListadoEquiposController.jpf'

        browser = mechanize.Browser()
        product_links = []

        baseData = browser.open(url).get_data()
        json_data = json.loads(baseData, strict=False)

        json_products = json_data['equipos']
        for json_product in json_products:
            product_id = json_product['codProducto']
            product_links.append(['http://personas.entel.cl/PortalPersonas/' \
                                  'appmanager/entelpcs/personas?_nfpb=true&' \
                                  '_pageLabel=P12800364521294253893661&' \
                                  'codProducto=' + product_id, 'Cell'])

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Movistar(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()

        try:
            product_data = browser.open(url).get_data()
        except mechanize.HTTPError:
            return None

        soup = BeautifulSoup(product_data)

        name = soup.find('h2').string.encode('ascii', 'ignore')

        prepago_price_container = soup.find('p',
                {'class': 'pnormalForeverAlone clearfix'})

        if prepago_price_container:
            price = prepago_price_container.find('strong').string
            price = Decimal(clean_price_string(price))
        else:
            price = Decimal(0)

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        browser = mechanize.Browser()

        if 'Cell' not in product_types:
            return []

        product_links = []
        page_number = 1
        while True:
            url = 'http://hogar.movistar.cl/equipos/index.php/catalogo' \
                  '/pagina/' + str(page_number)

            soup = BeautifulSoup(browser.open(url).get_data())

            prod_list = soup.findAll('div', {'class': 'producto'})

            if not prod_list:
                break

            for prod in prod_list:
                link = prod.find('a')['href']
                product_links.append([link, 'Cell'])

            page_number += 1

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class FullNotebook(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        try:
            product_data = browser.open(url).get_data()
        except Exception:
            return None
        product_soup = BeautifulSoup(product_data)

        product_name = product_soup.findAll('h2')[1].find('a').string
        product_name = product_name.encode('ascii', 'ignore')

        product_price = product_soup.find('span', {'id': 'esp'}).string
        product_price = Decimal(clean_price_string(product_price))

        return [product_name, product_price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.fullnotebook.cl/'
        url_buscar_productos = 'tienda/'

        browser = mechanize.Browser()

        product_links = []

        url_extensions = [
            ['notebooks', 'Notebook'],
            ['mini-notebooks', 'Notebook'],
            ['netbook', 'Notebook'],
        ]

        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            first_url = url_base + url_buscar_productos + url_extension + \
                        '/page/1'
            soup = BeautifulSoup(browser.open(first_url).get_data())

            lista_pags = soup.find('span', {'class': 'pages'})
            page_count = int(lista_pags.contents[0][-1])

            for i in range(page_count):
                page_url = url_base + url_buscar_productos + url_extension + \
                          '/page/' + str(i + 1)

                soup = BeautifulSoup(browser.open(page_url).get_data())

                for cont in soup.findAll('div', {'class': 'cliente'}):
                    elem = [cont.find('a')['href'].encode('ascii', 'ignore'),
                            ptype]
                    product_links.append(elem)

        return product_links



from decimal import Decimal
import unittest
from storescrapper.utils import count_decimals, format_currency
from storescrapper.utils import clean_price_string


class TestUtils(unittest.TestCase):

    def test_clean_price_string(self):
        value_expected_pairs = [['199.990', '199990'], ['990,990', '990990'],
            ['$19.990', '19990'], ['$105.500', '105500']]
        for dec_string, expected_value in value_expected_pairs:
            self.assertEqual(expected_value, clean_price_string(dec_string))

    def test_count_decimals(self):
        value_expected_pairs = [['123.45', 2], ['42', 0], ['-105.785', 3],
            ['-99', 0], ['87.', 0]]
        for decimal_string, expected_value in value_expected_pairs:
            value = Decimal(decimal_string)
            self.assertEqual(expected_value, count_decimals(value))

    def test_format_currency(self):
        value_expected_pairs = [['199990', '$199.990'], ['990', '$990'],
            ['29.95', '$29,95'], ['1010990', '$1.010.990'],
            ['-0.99', '-$0,99']]
        for decimal_string, expected_value in value_expected_pairs:
            value = Decimal(decimal_string)
            self.assertEqual(expected_value, format_currency(value))

    def test_init_tests(self):
        __import__('storescrapper.tests')



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class PcExpress(Store):

    @classmethod
    def _retrieve_product(cls, url):
        br = mechanize.Browser()
        data = br.open(url).get_data()
        soup = BeautifulSoup(data)

        name = soup.findAll('b')[2].string.encode('ascii', 'ignore')

        price = soup.find('font', {'size': '3'}).find('span').find('span')
        price = Decimal(clean_price_string(price.find('b').string))

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        browser = mechanize.Browser()

        url_extensions = [
            ['75_136', 'Notebook'],   # Notebooks
            ['83_157', 'VideoCard'],   # Tarjetas de video AGP
            ['83_158', 'VideoCard'],   # Tarjetas de video PCIe AMD
            ['83_159', 'VideoCard'],   # Tarjetas de video PCIe Nvidia
            ['61_85', 'Processor'],    # Procesadores AM2
            ['61_84', 'Processor'],    # Procesadores AM3
            ['61_86', 'Processor'],    # Procesadores 775
            ['61_193', 'Processor'],   # Procesadores 1155
            ['61_87', 'Processor'],    # Procesadores 1156
            ['61_165', 'Processor'],   # Procesadores 1366
            ['73_128', 'Monitor'],   # LCD
            ['73_129', 'Television'],   # LCD/TV
            ['73_171', 'Monitor'],   # LED
            ['60_88', 'Motherboard'],   # Placas madre AM2
            ['60_89', 'Motherboard'],   # Placas madre AM3
            ['60_90', 'Motherboard'],   # Placas madre 775
            ['60_194', 'Motherboard'],   # Placas madre 1155
            ['60_91', 'Motherboard'],   # Placas madre 1156
            ['60_186', 'Motherboard'],   # Placas madre 1366
            ['72_126', 'Ram'],   # RAM Desktop
            ['72_127', 'Ram'],   # RAM Notebook
            ['62_101', 'StorageDrive'],   # HDD PC
            ['62_103', 'StorageDrive'],   # HDD Notebook
            ['70_118', 'PowerSupply'],   # Fuentes de poder
        ]

        product_links = []

        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            url = 'http://www.pc-express.cl/index.php?cPath=' + \
                         url_extension

            soup = BeautifulSoup(browser.open(url).get_data())

            td_products = soup.findAll('td',
                    {'class': 'wrapper_pic_br wrapper_pic_td'})

            for td_product in td_products:
                link = td_product.find('a')['href']
                product_links.append([link, ptype])

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Rigam(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        product_soup = BeautifulSoup(browser.open(url).get_data())

        availability = product_soup.find('b', {'style': 'color:red;'})
        if availability:
            if 'PRODUCTO AGOTADO' in availability.string:
                return None

        name = product_soup.find('td', {'class': 'cy2'}).find('strong')
        name = name.contents[0].encode('ascii', 'ignore')

        price = product_soup.find('span', {'class': 'cy3'}).string
        price = Decimal(clean_price_string(price))

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.rigam.cl/catalogo/'
        browser = mechanize.Browser()

        url_extensions = [
            ['index.php?act=viewCat&catId=39', 'Notebook'],
            ['index.php?act=viewCat&catId=60', 'VideoCard'],
            ['index.php?act=viewCat&catId=40', 'Processor'],
            ['index.php?act=viewCat&catId=2', 'Processor'],
            ['index.php?act=viewCat&catId=54', 'Monitor'],
            ['index.php?act=viewCat&catId=3', 'Motherboard'],
            ['index.php?act=viewCat&catId=52', 'Motherboard'],
            ['index.php?act=viewCat&catId=35', 'Ram'],
            ['index.php?act=viewCat&catId=70', 'StorageDrive'],
            ['index.php?act=viewCat&catId=58', 'StorageDrive'],
            ['index.php?act=viewCat&catId=59', 'PowerSupply'],
        ]

        product_links = []
        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            page_number = 0

            while True:
                url = url_base + url_extension + '&page=' + \
                      str(page_number)

                baseSoup = BeautifulSoup(browser.open(url).get_data())

                products_table = baseSoup.find('table', {'class': 'tblList'})

                if not products_table:
                    break

                product_rows = products_table.findAll('tr')[1:]

                for product_row in product_rows:
                    link = product_row.find('a', {'class': 'txtDefault'})
                    product_links.append([url_base + link['href'], ptype])

                page_number += 1

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Paris(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        product_data = browser.open(url).get_data()
        soup = BeautifulSoup(product_data)

        name = soup.find('div', {'id': 'ficha-producto-nombre'}).string
        name = name.encode('ascii', 'ignore').strip()

        prices = []

        try:
            price = soup.find('div', {'id': 'ficha-producto-precio'})
            price = price.string.split('$')[1]
            price = Decimal(clean_price_string(price))
            prices.append(price)
        except Exception:
            pass

        try:
            price = soup.find('div', {'id': 'ficha-producto-precio-normal'})
            price = price.string.split('$')[1]
            price = Decimal(clean_price_string(price))
            prices.append(price)
        except Exception:
            pass

        if not prices:
            raise Exception

        price = min(prices)

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        browser = mechanize.Browser()

        urls = [
            # Notebooks
            ['http://www.paris.cl/webapp/wcs/stores/servlet/'
             'categoryTodos_10001_40000000577_-5_51145648_18877035_si_2__'
             '18877035,50999203,51145648_', 'Notebook'],
            # Netbooks
            ['http://www.paris.cl/webapp/wcs/stores/servlet/'
             'category_10001_40000000577_-5_51056269_18877035_51039699_'
             '18877035,51039699,51056269', 'Notebook'],
            # LCD
            ['http://www.paris.cl/webapp/wcs/stores/servlet/categoryTodos_'
             '10001_40000000577_-5_51056211_20096521_si_2__20096521,51056194,'
             '51056196,51056211_', 'Television'],
            ]

        product_links = []
        for url, ptype in urls:
            if ptype not in product_types:
                continue

            base_data = unicode(browser.open(url).get_data(), errors='ignore')
            soup = BeautifulSoup(base_data)

            link_divs = soup.findAll('div', {'class': 'descP2011'})

            for div in link_divs:
                linkId = int(div.find('a')['id'].replace('prod', '')) + 1

                link = 'http://www.paris.cl/webapp/wcs/stores/servlet/' \
                       'productLP_10001_40000000577_-5_51049202_18877035_' + \
                       str(linkId) + '_18877035,50999203,51049192,51049202__' \
                                     'listProd'

                product_links.append([link, ptype])

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class TopPc(Store):

    @classmethod
    def _retrieve_product(cls, url):
        br = mechanize.Browser()
        soup = BeautifulSoup(br.open(url).get_data())

        try:
            name = soup.find('h2').string
        except Exception:
            return None

        price = soup.find('span', {'id': 'our_price_display'}).string
        price = Decimal(clean_price_string(price))

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        browser = mechanize.Browser()

        url_extensions = [
            ['76', 'VideoCard'],    # Tarjetas de video
            ['5', 'Processor'],     # Procesadores
            ['61', 'Monitor'],    # Monitores y TV
            ['153', 'Notebook'],   # Notebooks
            ['8', 'Motherboard'],   # MB
            ['11', 'Ram'],   # RAM
            ['17', 'StorageDrive'],   # HDD SATA
            ['34', 'StorageDrive'],   # HDD IDE Desktop
            ['35', 'StorageDrive'],   # HDD IDE Notebook
        ]

        product_links = []

        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            url = 'http://www.toppc.cl/beta/category.php?n=50&id_category=' + \
                  url_extension

            soup = BeautifulSoup(browser.open(url).get_data())

            raw_links = soup.findAll('a', {'class': 'product_img_link'})

            for raw_link in raw_links:
                link = raw_link['href']
                product_links.append([link, ptype])

        return product_links



from storescrapper.stores.abcdin import AbcDin
from storescrapper.stores.bip import Bip
from storescrapper.stores.clie import Clie
from storescrapper.stores.dell import Dell
from storescrapper.stores.e_notebook import ENotebook
from storescrapper.stores.entel import Entel
from storescrapper.stores.falabella import Falabella
from storescrapper.stores.full_notebook import FullNotebook
from storescrapper.stores.global_mac import GlobalMac
from storescrapper.stores.hp_online import HpOnline
from storescrapper.stores.la_polar import LaPolar
from storescrapper.stores.mac_online import MacOnline
from storescrapper.stores.magens import Magens
from storescrapper.stores.movistar import Movistar
from storescrapper.stores.my_box import MyBox
from storescrapper.stores.notebook_center import NotebookCenter
from storescrapper.stores.packard_bell import PackardBell
from storescrapper.stores.paris import Paris
from storescrapper.stores.pc_express import PcExpress
from storescrapper.stores.pc_factory import PcFactory
from storescrapper.stores.pc_ofertas import PcOfertas
from storescrapper.stores.peta import Peta
from storescrapper.stores.portatil_chile import PortatilChile
from storescrapper.stores.rigam import Rigam
from storescrapper.stores.ripley import Ripley
from storescrapper.stores.sistemax import Sistemax
from storescrapper.stores.syd import Syd
from storescrapper.stores.sym import Sym
from storescrapper.stores.top_pc import TopPc
from storescrapper.stores.ttchile import TtChile
from storescrapper.stores.webco import Webco
from storescrapper.stores.wei import Wei

__all__ = [
    AbcDin.__name__,
    Bip.__name__,
    Clie.__name__,
    Dell.__name__,
    ENotebook.__name__,
    Entel.__name__,
    Falabella.__name__,
    FullNotebook.__name__,
    GlobalMac.__name__,
    HpOnline.__name__,
    LaPolar.__name__,
    MacOnline.__name__,
    Magens.__name__,
    Movistar.__name__,
    MyBox.__name__,
    NotebookCenter.__name__,
    PackardBell.__name__,
    Paris.__name__,
    PcExpress.__name__,
    PcFactory.__name__,
    PcOfertas.__name__,
    Peta.__name__,
    PortatilChile.__name__,
    Rigam.__name__,
    Ripley.__name__,
    Sistemax.__name__,
    Syd.__name__,
    Sym.__name__,
    TopPc.__name__,
    TtChile.__name__,
    Webco.__name__,
    Wei.__name__
]



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class PackardBell(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        product_data = browser.open(url).get_data()
        soup = BeautifulSoup(product_data)

        try:
            avail = int(soup.find('span', {'class': 'unidades'}).string)
            if not avail:
                return None
        except Exception:
            return None

        name = soup.find('div', {'class': 'tit_prod_det'}).string
        name = name.encode('ascii', 'ignore').strip()

        try:
            price = soup.find('div', {'class': 'precio_det'}).contents[1]
            price = Decimal(clean_price_string(price))
        except Exception:
            return None

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        browser = mechanize.Browser()

        url_extensions = [
            ['116-Netbook.html', 'Notebook'],
            ['112-Notebook.html', 'Notebook'],
        ]

        product_links = []
        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            url = 'http://www.packardbell.cl/2010/catalogo/' + url_extension
            soup = BeautifulSoup(browser.open(url).get_data())
            imgs = soup.findAll('div', {'class': 'img_prod'})

            for img in imgs:
                l = img['onclick'].replace('javascript:location.href=\'', '')
                l = l.replace('\'', '')
                product_links.append([l, ptype])

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class PcFactory(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        soup = BeautifulSoup(browser.open(url).get_data())

        name_container = soup.find('span', {'class': 'men_confirmacion'})
        name = name_container.string.encode('ascii', 'ignore').strip()

        price = soup.find('span',
                {'class': 'texto_Precio_Oferta_Internet_BIG'})
        price = Decimal(clean_price_string(price.string))

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.pcfactory.cl'

        browser = mechanize.Browser()

        url_extensions = [
            ['?papa=24&categoria=424', 'Notebook'],   # Notebooks 7 a 11
            ['?papa=24&categoria=449', 'Notebook'],   # Notebooks 12 a 13
            ['?papa=24&categoria=410', 'Notebook'],   # Notebooks 14
            ['?papa=24&categoria=437', 'Notebook'],   # Notebooks 15
            ['?papa=24&categoria=436', 'Notebook'],   # Notebooks 16 y +
            ['?papa=334&categoria=40', 'VideoCard'],   # VGA AGP
            ['?papa=334&categoria=378', 'VideoCard'],  # VGA PCIe Nvidia
            ['?papa=334&categoria=454', 'VideoCard'],  # VGA PCIe ATI
            ['?papa=334&categoria=455', 'VideoCard'],  # VGA Profesionales
            ['?papa=272&categoria=272', 'Processor'],
            ['?papa=256&categoria=250', 'Monitor'],  # Monitores LCD
            ['?papa=256&categoria=260', 'Television'],  # Televisores LCD
            ['?papa=292&categoria=292', 'Motherboard'],
            ['?papa=264&categoria=112', 'Ram'],  # Memoria PC
            ['?papa=264&categoria=482', 'Ram'],  # Memoria PC High-End
            ['?papa=264&categoria=100', 'Ram'],  # Memoria Notebook
            ['?papa=264&categoria=266', 'Ram'],  # Memoria Server
            ['?papa=312&categoria=340', 'StorageDrive'],  # HDD PC
            ['?papa=312&categoria=421', 'StorageDrive'],  # HDD Notebook
            ['?papa=326&categoria=54', 'PowerSupply'],  # Fuentes de poder
        ]

        product_links = {}
        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue

            url = url_base + '/' + url_extension

            soup = BeautifulSoup(browser.open(url).get_data())

            try:
                page_count = soup.findAll('table',
                        {'class': 'descripcionbold'})
                page_count = page_count[1].findAll('td', {'align': 'center'})
                page_count = int(page_count[-1].find('a').string)
            except Exception:
                page_count = 1

            for page in range(page_count):
                page += 1

                complete_url = url + '&pagina=' + str(page)

                soup = BeautifulSoup(browser.open(complete_url).get_data())

                product_link_tags = soup.findAll('a',
                        {'class': 'vinculoNombreProd'})

                for product_link in product_link_tags:
                    link = url_base + product_link['href']
                    link = link.encode('ascii', 'ignore')
                    if link not in product_links:
                        product_links[link] = ptype

        return product_links.items()



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Dell(Store):

    @classmethod
    def _retrieve_product(cls, url):
        br = mechanize.Browser()
        data = br.open(url).get_data()
        soup = BeautifulSoup(data)

        try:
            title = soup.find('div', {'id': 'scpcc_title'}).find('img')['alt']
            title = title.encode('ascii', 'ignore')
        except Exception:
            return None

        price = soup.find(['tr', 'td'], {'class': 'pricing_dotdotdot'})
        price = price.findAll('span')[-1].string.split('$')[1]
        price = Decimal(clean_price_string(price))

        return [title, price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        cookies = mechanize.CookieJar()
        opener = mechanize.build_opener(mechanize.HTTPCookieProcessor(cookies))
        opener.addheaders = [('User-agent', 'Mozilla/5.0 (MyProgram/0.1)'),
            ('From', 'responsible.person@example.com')]
        mechanize.install_opener(opener)

        url_buscar_productos = '/cl/'
        product_links = []
        url_base = 'http://www.dell.com'

        # Start home
        if 'Notebook' in product_types:
            url_extensions = [
                'p/laptops?avt=series&avtsub=inspiron-mini-netbooks&~ck='
                'supertab-inspiron-mini-netbooks',
            ]

            for url_extension in url_extensions:
                url_webpage = url_base + url_buscar_productos + url_extension

                r = mechanize.urlopen(url_webpage)
                base_soup = BeautifulSoup(r.read())

                tab_navigator = base_soup.find('div',
                        {'class': 'superTabarellaTabContainer'})
                line_urls = [url_base + link['href']
                             for link in tab_navigator.findAll('a')]

                for url in line_urls:
                    product_links.extend(cls.retrieve_line_links(url))

            # Start business

            url_extensions = [
                'empresas/p/laptops.aspx',
                ]

            for url_extension in url_extensions:
                url_webpage = url_base + url_buscar_productos + url_extension
                r = mechanize.urlopen(url_webpage)
                base_soup = BeautifulSoup(r.read())

                line_title_containers = base_soup.findAll('div',
                        {'class': 'productTitle'})
                line_urls = [url_base + div.find('a')['href']
                             for div in line_title_containers if div.find('a')]
                for url in line_urls:
                    product_links.extend(cls.retrieve_line_links(url))

        # Start Monitor

        if 'Monitor' in product_types:
            url_extensions = [
                '/content/products/compare.aspx/19_22widescreen'
                '?c=cl&cs=cldhs1&l=es&s=dhs',
                '/content/products/compare.aspx/23_30widescreen'
                '?c=cl&cs=cldhs1&l=es&s=dhs',
                '/cl/es/empresas/Monitores/19_15widescreen/cp.aspx'
                '?refid=19_15widescreen&s=bsd&cs=clbsdt1',
                '/cl/es/empresas/Monitores/22_20widescreen/cp.aspx'
                '?refid=22_20widescreen&s=bsd&cs=clbsdt1',
                '/cl/es/empresas/Monitores/30_24widescreen/cp.aspx'
                '?refid=30_24widescreen&s=bsd&cs=clbsdt1',
                '/cl/es/empresas/Monitores/20_19flatpanel/cp.aspx'
                '?refid=20_19flatpanel&s=bsd&cs=clbsdt1',
                ]

            for url_extension in url_extensions:
                url_webpage = url_base + url_extension

                r = mechanize.urlopen(url_webpage)
                base_soup = BeautifulSoup(r.read())

                links = base_soup.findAll('a', {'class': 'lnk'})
                for link in links:
                    if 'configure' in link['href']:
                        product_links.append([link['href'], 'Monitor'])

        return product_links

    @classmethod
    def retrieve_subline_links(cls, url):
        mecha = mechanize.urlopen(url)
        soup = BeautifulSoup(mecha.read())

        buttons = soup.findAll('div', {'class': 'buttons'})
        links = [button.find('a')['href'] for button in buttons]

        if not links:
            redirected_url = mecha.geturl()
            if 'configure' in redirected_url:
                final_links = [[redirected_url, 'Notebook']]
            else:
                final_links = []
        else:
            final_links = [[link, 'Notebook']
                for link in links if 'configure' in link]

        return final_links

    @classmethod
    def retrieve_line_links(cls, url):
        r = mechanize.urlopen(url)
        soup = BeautifulSoup(r.read())

        sub_line_containers = soup.findAll('div', {'class': 'infoCol'})
        sub_line_urls = [container.find('a')['href']
                         for container in sub_line_containers]
        fixed_urls = ['http://www.dell.com' + url[:-2] + 'fs'
                      for url in sub_line_urls]

        links = []
        for url in fixed_urls:
            links.extend(cls.retrieve_subline_links(url))

        return links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class HpOnline(Store):

    @classmethod
    def _retrieve_product(cls, url):
        br = mechanize.Browser()

        # Double open URL because the first redirects to home and sets a cookie
        BeautifulSoup(br.open(url).get_data())
        product_soup = BeautifulSoup(br.open(url).get_data())

        product_name = product_soup.findAll('h2')[-1].string
        product_name = product_name.encode('ascii', 'ignore')

        product_price = product_soup.find('h1', {'class': 'bigtitle'})
        product_price = product_price.contents[0].split('$')[1]
        product_price = Decimal(clean_price_string(product_price))

        return [product_name, product_price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.hponline.cl'

        browser = mechanize.Browser()

        url_extensions = [
            ['categoria.aspx?cat=ZA==&V=G', 'Notebook'],
        ]

        product_links = []
        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            url = url_base + '/personas/' + url_extension

            soup = BeautifulSoup(browser.open(url).get_data())
            product_cells = soup.findAll('div', {'class': 'product'})

            for cell in product_cells:
                elem = [url_base + cell.find('a')['href'],
                        ptype]
                product_links.append(elem)

        return product_links



from storescrapper.utils import format_currency


class Product(object):
    """
    Class that abstracts the information of a scrapped product
    """

    def __init__(self, name, price, url, type, store):
        """
        Constructor of the Product class

        :param name: Name of the product
        :type name: unicode
        :param price: Price of the product
        :type price: decimal.Decimal
        :param url: URL of the product in the store website
        :type url: unicode
        :param type: Type of the product
        :type type: unicode
        :param store: Name of the store where the product was found
        :type store: unicode
        """

        self.price = price
        self.name = name
        self.url = url
        self.type = type
        self.store = store

    def __repr__(self):
        return unicode(self)

    def __str__(self):
        return unicode(self)

    def __eq__(self, other):
        attrs = ['price', 'name', 'url', 'type', 'store']

        for attr in attrs:
            if getattr(self, attr) != getattr(other, attr):
                return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)

    def __unicode__(self):
        return '{0} - {1} ({2})\n{3}\n{4}'.format(self.store, self.name,
            self.type, self.url, format_currency(self.price))



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class ENotebook(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        product_data = browser.open(url).get_data()
        product_soup = BeautifulSoup(product_data)

        product_cells = product_soup.findAll('td', {'class': 'pageHeading'})

        try:
            product_name = product_cells[0].find('h1').contents[0]
        except IndexError:
            return None
        price_cell = product_cells[1]

        try:
            product_price = Decimal(clean_price_string(price_cell.find('span',
                    {'class': 'productSpecialPrice'}).string))
        except Exception:
            product_price = Decimal(clean_price_string(
                price_cell.string.split('$')[1]))

        return [product_name, product_price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.notebook.cl/'
        url_buscar_productos = 'venta/'

        browser = mechanize.Browser()

        url_extensions = [
            ['it-index-n-notebooks-cP-1.html', 'Notebook'],
        ]

        extra_pagelinks = [
            ['it-index-n-memoria_notebook-cP-46_4.html', 'Ram'],
            ['it-index-n-discos_duros-cP-46_23.html', 'StorageDrive'],
        ]

        page_links = []
        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            urlWebpage = url_base + url_buscar_productos + url_extension

            base_soup = BeautifulSoup(browser.open(urlWebpage).get_data())

            table_navigator = base_soup.findAll('table',
                    {'cellpadding': '2'})[1]

            pageNavigator = table_navigator.findAll('td',
                    {'class': 'smallText'})
            for pn in pageNavigator:
                link = pn.find('a')
                try:
                    page_links.append([link['href'].split('?osCsid')[0],
                                       ptype])
                except Exception:
                    continue

        for page_link, ptype in extra_pagelinks:
            if ptype not in product_types:
                continue
            page_links.append([url_base + url_buscar_productos + page_link,
                               ptype])

        product_link_pages = []
        for pageLink, ptype in page_links:
            links = cls.extract_product_pages(pageLink)
            for link in links:
                product_link_pages.append([link, ptype])

        product_links = []
        for product_link_page, ptype in product_link_pages:
            for link in cls.extract_product_links(product_link_page):
                product_links.append([link, ptype])

        return product_links

    @classmethod
    def extract_product_links(cls, page_link):
        br = mechanize.Browser()
        data = br.open(page_link).get_data()
        soup = BeautifulSoup(data)
        product_links = []
        cells = soup.findAll('td', {'class': 'productListing-data'})[1::4]
        for i in range(len(cells)):
            link = cells[i].find('a')
            product_links.append(link['href'].split('?osCsid')[0])
        return product_links

    @classmethod
    def extract_product_pages(cls, base_page_link):
        br = mechanize.Browser()
        data = br.open(base_page_link).get_data()
        soup = BeautifulSoup(data)
        links = [base_page_link]
        pageLinks = soup.findAll('a', {'class': 'pageResults'})[:-1]
        for pageLink in pageLinks:
            link = pageLink['href'].split('&osCsid')[0]
            links.append(link)

        return links



from decimal import Decimal
import json
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class LaPolar(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        product_data = browser.open(url).get_data()
        soup = BeautifulSoup(product_data)

        try:
            name = soup.find('div', {'class': 'LetraDetalleProducto'})
            name = name.string.encode('ascii', 'ignore')
        except AttributeError:
            return None

        price = soup.findAll('span', {'class': 'PrecioDetalleRojo'})[1]
        price = price.string.split('$')[1]
        price = Decimal(clean_price_string(price))

        return [name, price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        extensions = [
            ['electronica/computacion/notebook/', 'Notebook'],
            ['electronica/computacion/netbook/', 'Notebook'],
            ['electrohogar/tv_video/led/', 'Television'],
            ['electrohogar/tv_video/lcd/', 'Television'],
        ]

        browser = mechanize.Browser()

        products_data = []
        for extension, ptype in extensions:
            if ptype not in product_types:
                continue
            page = 1
            while True:
                url = 'http://www.lapolar.cl/internet/catalogo/listados/' + \
                      extension + str(page)
                baseSoup = BeautifulSoup(browser.open(url).get_data())

                try:
                    data = baseSoup.findAll('script')[-3].string.strip()
                    data = data.replace('var listado_productos_json = ', '')
                    data = json.loads(data[:-1])
                except ValueError:
                    break

                json_product_array_data = []

                for row in data['lista_completa']:
                    json_product_array_data.extend(row['sub_lista'])

                for json_product_data in json_product_array_data:
                    product_id = json_product_data['prid']

                    url = 'http://www.lapolar.cl/internet/catalogo/detalles/' \
                          '%s%s' % (extension, product_id)

                    products_data.append([url, ptype])

                page += 1

        return products_data



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class MacOnline(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        product_data = browser.open(url).get_data()
        soup = BeautifulSoup(product_data)

        name = soup.find('td', {'class': 'tit_categoria_despliegue'})
        name = name.contents[0].strip().encode('ascii', 'ignore')

        price = soup.find('em').string.split('$')[1]
        price = Decimal(clean_price_string(price))

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.maconline.cl'
        browser = mechanize.Browser()
        product_links = []

        url_extensions = [
            ['397.html', 'Notebook'],
            ['421.html', 'Notebook'],
            ['384.html', 'Notebook'],
            ['288.html', 'Monitor'],
            ['435.html', 'Notebook'],
            ['293', 'StorageDrive'],
        ]

        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            page_number = 0
            while True:
                url = url_base + '/catalogo/' + url_extension + \
                      '?pagina=' + str(page_number)

                soup = BeautifulSoup(browser.open(url).get_data())
                titles = soup.findAll('td', {'class': 'nombre_producto'})

                if not titles:
                    break

                for i in range(len(titles)):
                    link = url_base + titles[i].find('a')['href']
                    if link in product_links:
                        continue
                    product_links.append([link, ptype])
                page_number += 1

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class GlobalMac(Store):

    @classmethod
    def _retrieve_product(cls, url):
        cookies = mechanize.CookieJar()
        opener = mechanize.build_opener(mechanize.HTTPCookieProcessor(cookies))
        opener.addheaders = [('User-agent', 'Mozilla/5.0 (MyProgram/0.1)'),
            ('From', 'responsible.person@example.com')]
        mechanize.install_opener(opener)
        browser = mechanize.Browser()
        product_data = browser.open(url).get_data()
        product_soup = BeautifulSoup(product_data)

        product_name = product_soup.find('h1').string.encode('ascii', 'ignore')
        try:
            product_price = product_soup.find('span', {'id': 'product_price'})
            product_price = Decimal(clean_price_string(product_price.string))
        except Exception:
            return None

        return [product_name, product_price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        cookies = mechanize.CookieJar()
        opener = mechanize.build_opener(mechanize.HTTPCookieProcessor(cookies))
        opener.addheaders = [('User-agent', 'Mozilla/5.0 (MyProgram/0.1)'),
            ('From', 'responsible.person@example.com')]
        mechanize.install_opener(opener)
        url_base = 'http://www.globalmac.cl/'

        browser = mechanize.Browser()

        url_extensions = [
            ['MacBook/', 'Notebook'],
            ['MacBook-Pro/', 'Notebook'],
            ['Monitores-LCD/', 'Monitor'],
            ['Cinema-Display/', 'Monitor'],
            ['Disco-Duro-SATA-2.5/', 'StorageDrive'],
            ['Discos-Duros-ATA-2.5/', 'StorageDrive'],
            ['Discos-Duros-SATA/', 'StorageDrive'],
        ]

        if 'Ram' in product_types:
            memory_catalog_url = url_base + 'Memorias/'
            base_data = browser.open(memory_catalog_url).get_data()
            soup = BeautifulSoup(base_data)
            subcats = soup.findAll('span', {'class': 'subcategories'})
            for subcat in subcats:
                link = subcat.find('a')['href'].replace(url_base, '')
                url_extensions.append([link, 'Ram'])

        product_links = []

        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            base_data = browser.open(url_base + url_extension).get_data()
            soup = BeautifulSoup(base_data)

            titles = soup.findAll('a', {'class': 'product-title'})

            for title in titles:
                product_links.append([title['href'], ptype])

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class TtChile(Store):

    @classmethod
    def _retrieve_product(cls, url):
        base_data = mechanize.urlopen(url)
        base_soup = BeautifulSoup(base_data)

        image = base_soup.findAll('div', {'class': 'textOtrosPrecios'})[2]
        image = image.find('img')['src']
        if 'agotado' in image or 'proximo' in image:
            return None

        title = base_soup.find('div', {'class': 'textTituloProducto'})
        title = title.string.strip().encode('ascii', 'ignore')

        prices = base_soup.findAll('div', {'class': 'textPrecioContado'})
        price = prices[0].string
        price = Decimal(clean_price_string(price))

        return [title, price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.ttchile.cl/'
        product_urls_and_types = []

        url_extensions = [
            ['subpro.php?ic=21&isc=20', 'Notebook'],    # Notebooks
            ['catpro.php?ic=31', 'VideoCard'],          # Tarjetas de video
            ['catpro.php?ic=25', 'Processor'],          # Procesadores AMD
            ['catpro.php?ic=26', 'Processor'],          # Procesadores Intel
            ['catpro.php?ic=18', 'Monitor'],             # LCD
            ['catpro.php?ic=23', 'Motherboard'],        # MB AMD
            ['catpro.php?ic=24', 'Motherboard'],        # MB Intel
            ['subpro.php?ic=16&isc=10', 'Ram'],         # RAM DDR
            ['subpro.php?ic=16&isc=11', 'Ram'],         # RAM DDR2
            ['subpro.php?ic=16&isc=12', 'Ram'],         # RAM DDR3
            ['subpro.php?ic=16&isc=13', 'Ram'],         # RAM Notebook
            ['subpro.php?ic=10&isc=4', 'StorageDrive'],  # HDD IDE
            ['subpro.php?ic=10&isc=6', 'StorageDrive'],  # HDD Notebook
            ['subpro.php?ic=10&isc=5', 'StorageDrive'],  # HDD SATA
            ['subpro.php?ic=10&isc=7', 'StorageDrive'],  # SSD
            ['catpro.php?ic=12', 'PowerSupply'],        # Fuentes de poder
        ]

        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue

            page_number = 1

            while True:
                url = url_base + url_extension + '&pagina=' + \
                              str(page_number)
                base_data = mechanize.urlopen(url)
                base_soup = BeautifulSoup(base_data)

                divs = base_soup.findAll('div', {'class': 'linkTitPro'})
                product_links = [div.find('a')['href'] for div in divs]

                if not product_links:
                    break

                for product_link in product_links:
                    url = url_base + product_link
                    product_urls_and_types.append([url, ptype])

                page_number += 1

        return product_urls_and_types



import unittest
from storescrapper.exceptions.store_scrap_error import StoreScrapError
from storescrapper.store import Store


class TestStoreScriptsShouldRaiseErrorOnScrapError(unittest.TestCase):

    def setUp(self):
        class DummyStore(Store):
            @classmethod
            def _retrieve_product(cls, url):
                raise Exception('Forced exception')

            @classmethod
            def _product_urls_and_types(cls, product_types):
                raise Exception('Forced exception')
        self.store = DummyStore

    def test_retrieve_product(self):
        product_url = u'http://www.example.com/invalid_product'
        with self.assertRaises(StoreScrapError):
            self.store.retrieve_product(product_url, u'Notebook')

    def test_product_urls_and_types_with_async(self):
        with self.assertRaises(StoreScrapError):
            self.store.product_urls_and_types(product_types=[u'Notebook'])

    def test_product_urls_and_types_without_async(self):
        with self.assertRaises(StoreScrapError):
            self.store.product_urls_and_types(product_types=[u'StorageDrive'],
                async=False)

    def test_products_with_async(self):
        with self.assertRaises(StoreScrapError):
            self.store.products(product_types=[u'VideoCard'])

    def test_products_without_async(self):
        with self.assertRaises(StoreScrapError):
            self.store.products(product_types=[u'VideoCard'], async=False)



from decimal import Decimal
import unittest
from storescrapper.product import Product


class TestProduct(unittest.TestCase):
    def setUp(self):
        self.product = Product(u'Product1', Decimal(149990),
            u'http://www.example.com/product_1', u'Notebook', u'FooStore')

    def test_product_creation(self):
        self.assertEqual('Product1', self.product.name)
        self.assertEqual(Decimal(149990), self.product.price)
        self.assertEqual('http://www.example.com/product_1', self.product.url)
        self.assertEqual('Notebook', self.product.type)
        self.assertEqual('FooStore', self.product.store)

    def test_product_unicode(self):
        self.assertEqual('FooStore - Product1 (Notebook)\n'
                         'http://www.example.com/product_1\n$149.990',
            unicode(self.product))

    def test_product_str(self):
        self.assertEqual('FooStore - Product1 (Notebook)\n'
                         'http://www.example.com/product_1\n$149.990',
            str(self.product))

    def test_product_repr(self):
        self.assertEqual('FooStore - Product1 (Notebook)\n'
                         'http://www.example.com/product_1\n$149.990',
            repr(self.product))

    def test_product_equality(self):
        another_product = Product(u'Product1', Decimal(149990),
            u'http://www.example.com/product_1', u'Notebook', u'FooStore')
        self.assertEqual(another_product, self.product)

    def test_product_inequality(self):
        changed_attrs = [
            ['name', 'product1'],
            ['price', Decimal(150000)],
            ['url', 'http://example.com/product_1'],
            ['type', 'Notebooks'],
            ['store', 'BarStore'],
        ]

        for attr, new_value in changed_attrs:
            another_product = Product(u'Product1', Decimal(149990),
                u'http://www.example.com/product_1', u'Notebook', u'FooStore')
            setattr(another_product, attr, new_value)
            self.assertNotEquals(another_product, self.product)



import unittest


class TestImportingAllStoresShouldNotRaiseException(unittest.TestCase):

    def test_import_all_stores(self):
        try:
            __import__('storescrapper.stores', globals(), locals(), ['*'])
        except Exception, e:
            self.fail(e.message)



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store


class Sistemax(Store):

    force_sync = True

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        soup = BeautifulSoup(browser.open(url).get_data())

        name = soup.findAll('h2')[3].find('font').string
        name = name.encode('ascii', 'ignore')

        price = Decimal(soup.findAll('span', {'class': 'style18'})[0].string)

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        base_url = 'http://www.sistemax.cl/webstore/'

        extensions = [
            ['112', 'Notebook'],    # Netbooks
            ['27', 'Notebook'],     # Notebooks
            ['35', 'VideoCard'],    # Tarjetas de video
            ['58', 'Processor'],    # Procesadores AMD
            ['59', 'Processor'],    # Procesadores Intel
            ['25', 'Screen'],       # Monitores
            ['82', 'Notebook'],     # Mac / iPod
            ['30', 'Motherboard'],  # MB AMD
            ['69', 'Motherboard'],  # MB Intel
            ['24', 'Ram'],          # RAM
            ['21', 'PowerSupply'],  # Fuentes de poder
        ]

        product_links = []
        for extension, ptype in extensions:
            if ptype not in product_types:
                continue
            url = base_url + 'index.php?op=seccion/id=' + extension + \
                  '&page=-1&listar=true'

            browser = mechanize.Browser()

            soup = BeautifulSoup(browser.open(url).get_data())
            info_cells = soup.findAll('td', {'scope': 'col'})

            url_cells = info_cells[3::5]
            urls = [base_url + url_cell.contents[2]['href']
                    for url_cell in url_cells]
            for url in urls:
                product_links.append([url, ptype])

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class MyBox(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        product_data = browser.open(url).get_data()
        soup = BeautifulSoup(product_data)

        name = soup.find('h2').string
        price = soup.find('span', {'id': 'our_price_display'}).string
        price = Decimal(clean_price_string(price))

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.mybox.cl'
        browser = mechanize.Browser()

        url_extensions = [
            ['38-notebooks', 'Notebook'],
            ['7-monitores-y-proyectores', 'Monitor'],
            ['41-placas-madre', 'Motherboard'],
            ['44-procesadores', 'Processor'],
            ['54-tarjetas-de-video', 'VideoCard'],
            ['28-memoria-ram', 'Ram'],
            ['5-discos-duros', 'StorageDrive'],
            ['19-fuentes-de-poder', 'PowerSupply'],
        ]

        product_links = []
        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            url = url_base + '/' + url_extension + '?n=50'

            soup = BeautifulSoup(browser.open(url).get_data())
            prod_list = soup.find('ul', {'id': 'product_list'})

            if not prod_list:
                continue

            prod_cells = prod_list.findAll('li')

            for cell in prod_cells:
                entry = [url_base + cell.find('a')['href'], ptype]
                product_links.append(entry)

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Webco(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        product_soup = BeautifulSoup(browser.open(url).get_data())

        name = product_soup.find('h1').contents[0].encode('ascii', 'ignore')
        name = name.strip()

        price = product_soup.findAll('h2')[1].string.replace('cash', '')
        price = Decimal(clean_price_string(price))

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www1.webco.cl/'
        browser = mechanize.Browser()

        url_extensions = [
            # Netbooks
            ['n_new_productos.asp?CATEGORIA={761FD739-2D0F-4177-8AE0-'
             'C641D6F16502}', 'Notebook'],
            # Notebooks
            ['n_new_productos.asp?CATEGORIA={D70BBB30-F5E9-4246-B812-'
             'A939C8777429}', 'Notebook'],
            # Tarjetas de video
            ['n_new_productos.asp?CATEGORIA={FFE74755-6E24-4958-A066-'
             'F75670943D3E}', 'VideoCard'],
            # Procesadores AMD
            ['n_new_productos.asp?CATEGORIA={AA5D5535-B127-4AEE-8583-'
             '4529F66DE4D7}#ct_39', 'Processor'],
            # Procesadores Intel
            ['n_new_productos.asp?CATEGORIA={5701C20F-03E6-430E-8CCF-'
             '01EE820BEDF8}#ct_39', 'Processor'],
            # LCD TV
            ['n_new_productos.asp?CATEGORIA={E49A199E-214A-4658-99AA-'
             '7E6220434D8D}#ct_32', 'Television'],
            # LCD
            ['n_new_productos.asp?CATEGORIA={79A1AF72-4B4D-4368-9205-'
             'FC0D646A1145}#ct_32', 'Monitor'],
            # MB AMD
            ['n_new_productos.asp?CATEGORIA={C29A8E9A-5B73-4BEC-B6D7-'
             '665BA1B0D087}#ct_38', 'Motherboard'],
            # MB Intel
            ['n_new_productos.asp?CATEGORIA={9892E74D-67CD-4443-86F7-'
             '6FEBE141596B}#ct_38', 'Motherboard'],
            # HDD Notebook
            ['n_new_productos.asp?CATEGORIA={BCFA52D6-A416-4C48-BA90-'
             'F490B4F2C651}#ct_421', 'StorageDrive'],
            # HDD Notebook
            ['n_new_productos.asp?CATEGORIA={BCFA52D6-A416-4C48-BA90-'
             'F490B4F2C651}#ct_421', 'StorageDrive'],
            # HDD SATA
            ['n_new_productos.asp?CATEGORIA={80024ADB-0B06-4281-8CB6-'
             '7649D9E2C29B}#ct_421', 'StorageDrive'],
            # HDD IDE
            ['n_new_productos.asp?CATEGORIA={518AE532-05F0-4FEB-9FBE-'
             '3E67CA383E91}#ct_421', 'StorageDrive'],
            # RAM Desktop
            ['n_new_productos.asp?CATEGORIA={CB3ACADF-536F-4513-BD91-'
             '41E5E210B742}#ct_31', 'Ram'],
            # RAM Notebook
            ['n_new_productos.asp?CATEGORIA={B77F75B2-F280-4385-892A-'
             '01109D48ABD3}#ct_31', 'Ram'],
            ]

        product_links = {}
        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            url = url_base + url_extension
            soup = BeautifulSoup(browser.open(url).get_data())

            product_link_tags = soup.findAll('a')

            for link in product_link_tags:
                try:
                    url = url_base + link['href']
                    if url not in product_link_tags:
                        product_links[url] = ptype
                except Exception:
                    continue

        return product_links.items()



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
import re
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Falabella(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        product_data = browser.open(url).get_data()
        product_soup = BeautifulSoup(product_data)

        pn = product_soup.find('div',
                {'id': 'destacadoRuta'}).find('a').string

        if not pn:
            return None

        pn = pn.replace('&nbsp;', ' ').replace('\r', ' ').replace('\n', ' ')
        pn = ' '.join(re.split('\s+', pn.replace('\t', ' ')))
        product_name = pn.encode('ascii', 'ignore')

        product_price = product_soup.find('div', {'class': 'precio1'})
        product_price = Decimal(clean_price_string(product_price.contents[2]))

        return [product_name, product_price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        browser = mechanize.Browser()

        url_schemas = [
            ['http://www.falabella.com/falabella-cl/browse/productList.jsp?'
             '_dyncharset=iso-8859-1&requestChainToken=0005653415&pageSize=16&'
             'priceFlag=&categoryId=cat70057&docSort=numprop&docSortProp=price'
             '&docSortOrder=ascending&onlineStoreFilter=online'
             '&userSelectedFormat=4*4&trail=SRCH%3Acat70057&navAction=jump'
             '&searchCategory=true&question=cat70057'
             '&qfh_s_s=submit&_D%3Aqfh_s_s=+'
             '&qfh_ft=SRCH%3Acat70057&_D%3Aqfh_ft=+'
             '&_DARGS=%2Ffalabella-cl%2Fbrowse%2FfacetsFunctions.jsp'
             '&goToPage=', 'Notebook'],
            ['http://www.falabella.com/falabella-cl/browse/productList.jsp?'
             '_dyncharset=iso-8859-1&requestChainToken=0005655243&pageSize=16&'
             'priceFlag=&categoryId=cat70043&docSort=numprop&docSortProp=price'
             '&docSortOrder=ascending&onlineStoreFilter=online'
             '&userSelectedFormat=4*4&trail=SRCH%3Acat70043&navAction=jump'
             '&searchCategory=true&question=cat70043&qfh_s_s=submit'
             '&_D%3Aqfh_s_s=+&qfh_ft=SRCH%3Acat70043&_D%3Aqfh_ft=+'
             '&_DARGS=%2Ffalabella-cl%2Fbrowse%2FfacetsFunctions.jsp'
             '&goToPage=', 'Television'],
            ['http://www.falabella.com/falabella-cl/browse/productList.jsp?'
             '_dyncharset=iso-8859-1&requestChainToken=0005655389&pageSize=16&'
             'priceFlag=&categoryId=cat2053&docSort=numprop&docSortProp=price'
             '&docSortOrder=ascending&onlineStoreFilter=online'
             '&userSelectedFormat=4*4&trail=SRCH%3Acat2053&navAction=jump'
             '&searchCategory=true&question=cat2053&qfh_s_s=submit'
             '&_D%3Aqfh_s_s=+&qfh_ft=SRCH%3Acat2053&_D%3Aqfh_ft=+'
             '&_DARGS=%2Ffalabella-cl%2Fbrowse%2FfacetsFunctions.jsp'
             '&goToPage=', 'Television'],
            ['http://www.falabella.com/falabella-cl/browse/productList.jsp?'
             '_dyncharset=iso-8859-1&requestChainToken=0005655389&pageSize=16&'
             'priceFlag=&categoryId=cat70044&docSort=numprop&docSortProp=price'
             '&docSortOrder=ascending&onlineStoreFilter=online'
             '&userSelectedFormat=4*4&trail=SRCH%3Acat70044&navAction=jump'
             '&searchCategory=true&question=cat70044&qfh_s_s=submit'
             '&_D%3Aqfh_s_s=+&qfh_ft=SRCH%3Acat70044&_D%3Aqfh_ft=+'
             '&_DARGS=%2Ffalabella-cl%2Fbrowse%2FfacetsFunctions.jsp'
             '&goToPage=', 'Television'],

        ]
        product_links = []

        for url_schema, ptype in url_schemas:
            if ptype not in product_types:
                continue
            page_number = 1

            while True:
                url = url_schema + str(page_number)

                baseData = browser.open(url).get_data()
                baseSoup = BeautifulSoup(baseData)

                mosaicDivs = baseSoup.findAll('div', {'class': 'quickView'})

                if not mosaicDivs:
                    break

                for div in mosaicDivs:
                    url = 'http://www.falabella.com' + div.find('a')['href']
                    url = url.replace(' ', '')

                    m = re.search(';jsessionid=\S+\.node\d', url)
                    if m:
                        url = url[:m.start()] + url[m.end():]

                    url = ''.join(url.split())
                    product_links.append([url, ptype])

                page_number += 1

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class NotebookCenter(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        product_data = browser.open(url).get_data()
        soup = BeautifulSoup(product_data)

        name = soup.find('span', {'class': 'titulo_producto'}).string
        name = name.encode('ascii', 'ignore')

        price = soup.find('span', {'class': 'precio_producto_efectivo'})
        price = Decimal(clean_price_string(price.string))

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.notebookcenter.cl/'
        browser = mechanize.Browser()

        url_extensions = [
            ['565', 'Notebook'],  # Macbook Air
            ['558', 'Notebook'],  # Macbook Pro
            ['61', 'Notebook'],   # Notebook Acer
            ['251', 'Notebook'],  # Notebook Dell
            ['57', 'Notebook'],   # Notebook HP
            ['64', 'Notebook'],   # Notebook Lenovo
            ['418', 'Notebook'],  # Netbook Samsung
            ['212', 'Notebook'],  # Notebook Sony
            ['63', 'Notebook'],   # Notebook Toshiba
            ['603', 'Notebook'],   # Notebook Corporativo Acer
            ['602', 'Notebook'],   # Notebook Corporativo Lenovo
            ['598', 'Notebook'],   # Notebook Corporativo HP
            ['606', 'Notebook'],   # Notebook Corporativo Dell
            ['401', 'Notebook'],  # Netbook Acer
            ['402', 'Notebook'],  # Netbook Dell
            ['406', 'Notebook'],  # Netbook Samsung
            ['472', 'Processor'],  # Procesadores AMD
            ['473', 'Processor'],  # Procesadores Intel
            ['275', 'Monitor'],  # Monitores Apple
            ['162', 'Monitor'],  # Monitores LCD
            ['640', 'Monitor'],  # Monitores LED
            ['323', 'Television'],  # Televisor LCD
            ['641', 'Television'],  # Televisor LED
            ['6', 'Ram'],  # Notebook RAM
            ['109', 'StorageDrive'],  # HDD Notebook
            ['412', 'StorageDrive'],  # HDD Desktop
        ]

        product_links = []
        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue

            index = 0
            while True:
                url = url_base + '?p=1&op=2&p3=' + url_extension + '&i_p=' + \
                      str(index)

                soup = BeautifulSoup(browser.open(url).get_data())

                product_containers = soup.findAll('div',
                        {'id': 'producto_inicio'})
                if not product_containers:
                    break

                for product_container in product_containers:
                    js_data = product_container.find('a')['href']

                    js_ars = [int(s) for s in js_data.replace("'", ' ').split()
                              if s.isdigit()]
                    id_prod = js_ars[2]
                    link = url_base + '?p=2&op=1&i_p=' + str(id_prod)
                    product_links.append([link, ptype])

                index += 1

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Sym(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        soup = BeautifulSoup(browser.open(url).get_data())

        stock_status = soup.find('li', {'class': 'stocks'})
        stock_status = stock_status.findAll('span')[1].contents[0]

        if 'pedido' not in stock_status and 'stock' not in stock_status:
            return None

        title_span = soup.find('h1')
        title = str(title_span.string).strip()

        price_container = soup.find('span', {'class': 'red'})
        price_string = price_container.find('strong').string
        price = Decimal(clean_price_string(price_string))

        return [title, price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        browser = mechanize.Browser()

        url_extensions = [
            ['?cat=104', 'Notebook'],         # Notebooks
            ['?cat=32_68', 'VideoCard'],      # Tarjetas de video AGP
            ['?cat=32_69', 'VideoCard'],      # Tarjetas de video PCIe
            ['?cat=50', 'Processor'],         # Procesadores Intel
            ['?cat=25', 'Processor'],         # Procesadores AMD
            ['?cat=81', 'Monitor'],            # Monitores LCD
            ['?cat=26', 'Motherboard'],       # MB AMD
            ['?cat=49', 'Motherboard'],       # MB Intel
            ['?cat=27', 'Ram'],               # RAM
            ['?cat=29_61', 'StorageDrive'],   # HDD Notebook
            ['?cat=29_62', 'StorageDrive'],   # HDD SATA
            ['?cat=29_129', 'StorageDrive'],  # SSD
        ]

        product_urls_and_types = {}

        for url_extension, product_type in url_extensions:
            if product_type not in product_types:
                continue

            page_number = 1

            while True:
                url_webpage = 'http://www.sym.cl/%s&page=%d' % \
                             (url_extension, page_number)

                base_data = browser.open(url_webpage).get_data()
                base_soup = BeautifulSoup(base_data)

                link_containers = base_soup.findAll('div',
                        {'class': 'listadoindiv'})

                if not link_containers:
                    break

                break_flag = False

                for link_container in link_containers:
                    url = link_container.find('h2').find('a')['href']
                    if url in product_urls_and_types:
                        break_flag = True
                        break
                    product_urls_and_types[url] = product_type

                if break_flag:
                    break

                page_number += 1

        return product_urls_and_types.items()



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Peta(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        soup = BeautifulSoup(browser.open(url).get_data())

        try:
            avail = soup.find('p', {'class': 'availability in-stock'})
            avail = avail.find('span')
            if avail.string and avail.string != 'En existencia':
                return None
        except Exception:
            return None

        try:
            name = soup.find('h1', {'class': 'p-title'}).string
            name = name.encode('ascii', 'ignore')

            price = soup.find('span', {'class': 'price'}).string.split('$')[1]
            price = Decimal(clean_price_string(price))
        except Exception:
            return None

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        browser = mechanize.Browser()

        url_extensions = [
            ['computadores-1/netbooks.html', 'Notebook'],
            ['computadores-1/notebooks.html', 'Notebook'],
            ['computadores-1/apple.html?appletype=898,903', 'Notebook'],
            ['peta-cl/tarjetas-de-video.html', 'VideoCard'],
            ['peta-cl/procesadores.html', 'Processor'],
            ['peta-cl/monitores.html', 'Monitor'],
            ['audio-y-video-1/televisores.html', 'Television'],
            ['peta-cl/placas-madre-1.html', 'Motherboard'],
            ['partes-y-piezas/memorias.html', 'Ram'],
            ['partes-y-piezas/discos-duros.html', 'StorageDrive'],
            ['partes-y-piezas/fuentes-de-poder.html', 'PowerSupply'],
        ]

        product_links = []
        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue

            url = 'http://www.peta.cl/' + url_extension
            first_page_url = url + '?limit=36&p=1'

            soup = BeautifulSoup(browser.open(first_page_url).get_data())

            page_count = soup.find('div', {'class': 'pages'})
            if page_count:
                page_count = int(page_count.findAll('a')[-2].string)
            else:
                page_count = 1

            for page_number in range(page_count):
                page_number += 1
                complete_url = url + '?limit=36&p=' + str(page_number)

                soup = BeautifulSoup(browser.open(complete_url).get_data())
                soup = soup.find('div', 'category-products')

                p_cells = []
                p_cells.extend(soup.findAll('li', {'class': 'item first'}))
                p_cells.extend(soup.findAll('li', {'class': 'item'}))
                p_cells.extend(soup.findAll('li', {'class': 'item last'}))

                for cell in p_cells:
                    link = cell.find('a')['href']

                    product_links.append([link, ptype])

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Syd(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        soup = BeautifulSoup(browser.open(url).get_data())

        name = soup.findAll('h2')[5].string.encode('ascii', 'ignore')

        try:
            price = soup.find('div', {'class': 'detallesCompra'})
            price = price.findAll('dd')[1].string
            price = Decimal(clean_price_string(price))
        except Exception:
            return None

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.syd.cl'
        browser = mechanize.Browser()

        url_extensions = [
            ['/computadoras/macbook_pro', 'Notebook'],
            ['/computadoras/macbook', 'Notebook'],
            ['/computadoras/macbook_air', 'Notebook'],
            ['/computadoras/monitores', 'Monitor'],
            ['/memorias', 'Ram'],
        ]

        product_links = []
        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue

            url = url_base + url_extension + '/?op=all&crit='

            baseSoup = BeautifulSoup(browser.open(url).get_data())

            titles = baseSoup.findAll('h4')

            for title in titles:
                link = title.find('a')
                link = url_base + url_extension + '/' + link['href']
                product_links.append([link, ptype])

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Magens(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        try:
            product_data = browser.open(url).get_data()
        except Exception:
            return None
        product_soup = BeautifulSoup(product_data)

        try:
            availability = product_soup.find('div', {'class': 'stock'})
            availability = availability.contents[4]
        except AttributeError:
            return None

        if 'Agotado' in availability:
            return None

        name = product_soup.find('div', {'class': 'titleContent'}).string
        name = name.encode('ascii', 'ignore')

        price = product_soup.findAll('div', {'class': 'precioDetalle'})[1]
        price = price.string.split('$')[1]
        price = Decimal(clean_price_string(price))

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        browser = mechanize.Browser()

        url_extensions = [
            ['notebooks-netbooks-netbooks-11-c-15_199.html', 'Notebook'],
            ['notebooks-netbooks-notebooks-12-13-c-15_202.html', 'Notebook'],
            ['notebooks-netbooks-notebooks-14-c-15_203.html', 'Notebook'],
            ['notebooks-netbooks-notebooks-15-c-15_204.html', 'Notebook'],
            ['notebooks-netbooks-notebooks-16-mas-c-15_205.html', 'Notebook'],
            ['video-c-24_128.html', 'VideoCard'],
            ['video-pcie-c-24_130.html', 'VideoCard'],
            ['video-pcie-nvidia-c-24_129.html', 'VideoCard'],
            ['video-profesionales-c-24_131.html', 'VideoCard'],
            ['sam2-c-1_196.html', 'Processor'],
            ['sam3-c-1_31.html', 'Processor'],
            ['intel-s1156-c-1_197.html', 'Processor'],
            ['intel-s775-c-1_32.html', 'Processor'],
            ['server-c-1_30.html', 'Processor'],
            ['monitores-lcd-c-13_90.html', 'Monitor'],
            ['monitores-led-c-13_200.html', 'Monitor'],
            ['televisores-lcdtv-c-13_91.html', 'Television'],
            ['televisores-ledtv-c-13_210.html', 'Television'],
            ['placas-madre-c-2.html', 'Motherboard'],
            ['memorias-c-12.html', 'Ram'],
            ['discos-duros-c-3_35.html', 'StorageDrive'],
            ['discos-notebook-c-3_37.html', 'StorageDrive'],
            ['ssd-c-3_215.html', 'StorageDrive'],
            ['fuentes-poder-c-10_76.html', 'PowerSupply'],
        ]

        product_links = []
        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            url = 'http://www.magens.cl/catalog/' + url_extension + \
                  '?mostrar=1000'
            soup = BeautifulSoup(browser.open(url).get_data())

            product_containers = soup.findAll('div',
                    {'class': 'text11 uppercase tituloProducto'})
            for container in product_containers:
                link = container.find('a')['href']
                product_links.append([link.split('?osCsid')[0], ptype])

        return product_links



import logging
import traceback
from celery.task import task
from celery.task.sets import TaskSet
from storescrapper.exceptions.store_scrap_error import StoreScrapError
from storescrapper.product import Product


class Store(object):
    """
    Common superclass of all the scrap store scripts

    Implements the APIs to conveniently scrap the products of the available
    stores in storecrapper.stores using synchronous methods or
    asynchronous tasks.
    """

    ####################
    # API of the class #
    ####################

    @classmethod
    def products(cls, product_types, async=True):
        """
        Retrieves the available products in the store that match one
        of the given product types.

        Returns a dictionary of unicode => Product matching the URL of the
        product with its complete data. If the product is unavailable the value
        of the pairing is None.

        :param product_types: List of product types
        :type product_types: list of unicode
        :param async: True to use asynchronous tasks
        :type async: bool

        :rtype: dict of storescrapper.product.Product

        """

        # If the store can't handle concurrent connections, force sync
        if hasattr(cls, 'force_sync') and getattr(cls, 'force_sync'):
            async = False

        logging.info('Retrieving products from {0} with types {1}'.format(
            cls.__name__, product_types))

        # Retrieve the URLs of the products
        product_urls_and_types = cls.product_urls_and_types(product_types,
            async=async)
        products = {}

        if async:
            logging.info('Using asynchronous tasks')
            # Dispatch a task for each of the URLs retrieved previously
            tasks = []
            func = cls.retrieve_product
            for url, product_type in product_urls_and_types:
                logging.info('Dispatching task for product with URL {0} ' \
                             'and type {1}'.format(url, product_type))
                tasks.append(func.subtask((cls, url, product_type)))

            job = TaskSet(tasks)
            job_result = job.apply_async()

            logging.info('Waiting for tasks to finish')

            for idx, product in enumerate(job_result.join()):
                logging.info('Result for product with URL: {0}'.format(
                    product_urls_and_types[idx][0]))
                if product:
                    logging.info(product)
                else:
                    logging.info('Not available')

                products[product_urls_and_types[idx][0]] = product

        else:
            logging.info('Using sync method')

            # Get the products one by one
            logging.info('Obtaining products information')
            for url, product_type in product_urls_and_types:
                product = cls.retrieve_product(url, product_type)

                if product:
                    logging.info(product)
                else:
                    logging.info('Not available')

                products[url] = product

        logging.info('Finished fetching products')
        return products

    @classmethod
    def product_urls_and_types(cls, product_types, async=True):
        """
        Returns a list of tuples of the form (url, product_type)
        corresponding to the basic data of the products of the store that
        match the given product types.

        :param product_types: List of product types
        :type product_types: list of unicode
        :param async: True to use asynchronous tasks
        :type async: bool

        :rtype: list of (unicode, unicode)

        """

        # If the store can't handle concurrent connections, force sync
        if hasattr(cls, 'force_sync') and getattr(cls, 'force_sync'):
            async = False

        logging.info('Retrieving product URLs from {0} with types {1}' \
                     ''.format(cls.__name__, product_types))
        if async:
            logging.info('Using async tasks')
            # We optimize the process by creating a task for each product type
            # and dispatching them, as the vast majority of the store classes
            # are faster when dealing with only one product type.
            tasks = []
            func = cls._async_product_urls_and_types
            for product_type in product_types:
                logging.info('Creating task for product type ' \
                             '{0}'.format(product_type))
                tasks.append(func.subtask((cls, product_type)))

            job = TaskSet(tasks)
            job_result = job.apply_async()

            results = []

            logging.info('Waiting for tasks to finish')
            for job_subresult in job_result.join():
                for result in job_subresult:
                    logging.info('Found product URL: {0}'.format(
                        result[0]))
                results.extend(job_subresult)

            logging.info('Finished getting product URLs from ' \
                         '{0}'.format(cls.__name__))

            return results

        else:
            # If the method is called without the async flag we just call the
            # underlying implementation and return the required result.
            logging.info('Using sync method')

            results = [(url, product_type)
                for url, product_type
                in cls.__product_urls_and_types(product_types)]

            for result in results:
                logging.info('Found product URL: {0}'.format(result[0]))

            logging.info('Finished getting product URLs')
            return results

    @classmethod
    @task
    def retrieve_product(cls, url, product_type):
        """
        Obtains the Product corresponding to the given URL or None if it is not
        available.

        :param url: URL of the product in the store
        :type url: unicode
        :param product_type: Product type
        :type product_type: unicode

        :rtype: Product

        """

        logging.info('Retrieving product from {0} with URL {1}'.format(
            cls.__name__, url))

        # Get the product with the underlying implementation
        try:
            product_info = cls._retrieve_product(url)
        except NotImplementedError:
            raise
        except Exception, e:
            message = 'Fetching of product with URL {0} raised '\
                      'exception: {1}'.format(url, e.message)
            logging.error(message)
            formatted_lines = traceback.format_exc().splitlines()
            logging.error(formatted_lines)
            raise StoreScrapError(message)

        # If it is available, load the metadata
        if product_info:
            p = Product(product_info[0], product_info[1], url,
                product_type, unicode(cls.__name__))

            logging.info('Product is available:\n{0}'.format(p))
            return p
        else:
            logging.info('Product is not available')
            return None

    #################################
    # Internal methods of the class #
    #################################

    @classmethod
    @task
    def _async_product_urls_and_types(cls, product_type):
        """
        Retrieves a list of tuples of the form (url, product_type)
        corresponding to the basic data of the products of the store that
        match the given product type.

        In particular this method includes the products that are not
        available in the store but still listed in their webpage.

        :param product_type: Product type
        :type product_type: unicode

        :rtype: list of (unicode, unicode)

        """

        logging.info('Retrieving product URLs from {0} with type {1}'.format(
            cls.__name__, product_type))

        product_urls_and_types = []

        for url, ptype in cls.__product_urls_and_types([product_type]):
            logging.info('Found product URL: {0}'.format(url))
            data = (url, ptype)
            product_urls_and_types.append(data)

        logging.info('Finished getting product URL from {0} with type ' \
                    '{1}'.format(cls.__name__, product_type))

        return product_urls_and_types

    @classmethod
    def __product_urls_and_types(cls, product_types):
        """
        Thin wrapper over the _product_urls_and_types method implemented
        for each store. It only checks for errors and wraps them with some
        metadata and a common exception interface.

        :param product_types: Product types
        :type product_types: list of unicode

        :rtype: list of (unicode, unicode)

        """
        try:
            return cls._product_urls_and_types(product_types)
        except NotImplementedError:
            raise
        except Exception, e:
            message = 'Error while fetching the products from '\
                      '{0} with types {1}: {2}'.format(cls.__name__,
                         product_types, e.message)
            logging.error(message)
            formatted_lines = traceback.format_exc().splitlines()
            logging.error(formatted_lines)
            raise StoreScrapError(message)

    #####################################################################
    # The following methods must be provided by the subclasses of Store #
    #####################################################################

    @classmethod
    def _product_urls_and_types(cls, product_types):
        """
        Returns a list of tuples, each corresponding to the basic information
        (URL and type) of each of the products in the store that match one
        of the given product types.

        :param product_types: List of product types
        :type product_types: list of unicode

        :rtype: list of (unicode, unicode)

        """

        raise NotImplementedError('The subclass of Store should implement '
                                  '_product_urls_and_types and Store should '
                                  'not be instantiated directly')

    @classmethod
    def _retrieve_product(cls, url):
        """
        Returns a list [name, price] corresponding to the name and price of
        the product with the given URL or None if it is not available.

        :param url: URL of the product in the store webpage
        :type url: unicode

        :rtype: (unicode, decimal.Decimal)

        """

        raise NotImplementedError('The subclass of Store should implement '
                                  '_retrieve_product and Store should not be '
                                  'instantiated directly')



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class PcOfertas(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        soup = BeautifulSoup(browser.open(url).get_data())

        try:
            name = soup.find('h4', {'class': 'Estilo5'}).string
            name = name.encode('ascii', 'ignore')
        except Exception:
            return None

        try:
            price = soup.find('span',
                    {'style': 'color: #F00; font-size:12px;'})
            price = price.parent.parent.parent.findAll('td')[1].find('span')
            price = Decimal(clean_price_string(price.string))
        except Exception:
            return None

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        product_links = []

        url_extensions = [
            ['74', 'Notebook'],   # Notebook
            ['75', 'Notebook'],   # Netbook
            ['87', 'VideoCard'],   # Tarjetas de video
            ['18', 'Processor'],   # Procesadores
            ['28', 'Monitor'],   # Monitores
            ['108', 'Motherboard'],   # Placas madre
            ['17', 'Ram'],   # RAM
            ['20', 'StorageDrive'],   # Almacenamiento
            ['72', 'PowerSupply'],   # Fuentes de poder
        ]

        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            url = 'http://www.pcofertas.cl/index.php?route=product/' \
                  'category&path=' + url_extension
            links = cls.recursive_retrieve_product_links(url)

            links = list(set(links))

            for link in links:
                base_link, args = link.split('?')
                args = dict([elem.split('=') for elem in args.split('&')])
                del args['path']
                args = ['%s=%s' % (k, v) for k, v in args.items()]
                link = base_link + '?' + '&'.join(args)
                if link not in product_links:
                    product_links.append([link, ptype])

        return product_links

    @classmethod
    def recursive_retrieve_product_links(cls, url):
        browser = mechanize.Browser()
        soup = BeautifulSoup(browser.open(url).get_data())
        prod_links = []

        products_table = soup.findAll('table', {'class': 'list'})
        if not products_table:
            return []
        elif len(products_table) == 2:
            # First get the products on the page
            products_anchor_tags = products_table[1].findAll('a')[::3]
            for anchor_tag in products_anchor_tags:
                link = anchor_tag['href']
                prod_links.append(link)

            # Then the ones on sub categories
            category_links = products_table[0].findAll('a')[::2]
            for anchor_tag in category_links:
                link = anchor_tag['href']
                prod_links.extend(cls.recursive_retrieve_product_links(link))
        else:
            # Page may have only products or only category links
            products_anchor_tags = products_table[0].findAll('a')[::3]
            for anchor_tag in products_anchor_tags:
                link = anchor_tag['href']
                if 'product_id' in link:
                    prod_links.append(link)
                else:
                    prod_links.extend(cls.recursive_retrieve_product_links(
                        link))
        return prod_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Wei(Store):

    @classmethod
    def _retrieve_product(cls, url):
        br = mechanize.Browser()
        soup = BeautifulSoup(br.open(url).get_data())

        availabilities = soup.find('table', {'class': 'pdisponibilidad'})
        availabilities = availabilities.findAll('td')

        for availability in availabilities:
            if 'Producto agotado' in availability.contents[1]:
                return None

        name = soup.find('title').string.split('WEI CHILE S. A. - ')[1]

        price = soup.find('table', {'class': 'pprecio'}).find('h1').string
        price = Decimal(clean_price_string(price))

        return name, price

    @classmethod
    def _product_urls_and_types(cls, product_types):
        base_url = 'http://www.wei.cl/'
        browser = mechanize.Browser()

        category_urls = [
            ['252', 'Notebook'],      # Notebooks
            ['175', 'VideoCard'],     # Tarjetas de video AGP
            ['176', 'VideoCard'],     # Tarjetas de video PCI Express
            ['119', 'Processor'],     # Procesadores AMD
            ['120', 'Processor'],     # Procesadores Intel
            ['205', 'Television'],    # LCD TV
            ['80', 'Monitor'],     # Monitores LCD
            ['65', 'Motherboard'],     # MB AMD
            ['84', 'Motherboard'],     # MB Intel
            ['68', 'Ram'],     # RAM Notebook
            ['89', 'Ram'],     # RAM DDR
            ['195', 'Ram'],     # RAM DDR2
            ['199', 'Ram'],     # RAM DDR3
            ['70', 'StorageDrive'],     # HDD IDE
            ['71', 'StorageDrive'],     # HDD SATA
            ['78', 'StorageDrive'],     # HDD Notebook
        ]

        product_links = {}
        for category_url, ptype in category_urls:
            if ptype not in product_types:
                continue
            desde = 1
            while True:
                url = base_url + 'index.htm?op=categoria&ccode=' + \
                      category_url + '&desde=' + str(desde)

                soup = BeautifulSoup(browser.open(url).get_data())

                product_cells = soup.findAll('div', {'class': 'box1'})
                flag = False

                if not product_cells:
                    break

                for cell in product_cells:
                    link = base_url + cell.parent['href']
                    if link in product_links:
                        flag = True
                        break
                    product_links[link] = ptype

                if flag:
                    break

                desde += 20

        return product_links.items()



from decimal import Decimal
import unittest
from storescrapper.product import Product
from storescrapper.store import Store


class TestStoreScriptsShouldReturnCorrectResults(unittest.TestCase):

    def setUp(self):
        class DummyStore(Store):
            @classmethod
            def _retrieve_product(cls, url):
                if 'not-available' in url:
                    return None

                d = dict(joined_key_value.split('=') for joined_key_value in
                    url.split('?')[1].split('&'))

                name = d['product']
                price = Decimal(d['price'])

                return [name, price]

            @classmethod
            def _product_urls_and_types(cls, product_types):
                data = [
                    ('http://dummystore.com/?product=dm1-3060&price=199990',
                     'Notebook'),
                    ('http://dummystore.com/?product=sapphire-hd6850&'
                     'availability=not-available&price=105500', 'VideoCard'),
                    ('http://dummystore.com/?product='
                     'intel-core-i5-2500k&price=145500', 'Processor'),
                ]

                result = []

                for url, type in data:
                    if product_types and type in product_types:
                        result.append([url, type])

                return result
        self.store = DummyStore

    def test_product_urls_and_types_with_async(self):
        product_urls_and_types = self.store.product_urls_and_types(
            [u'Notebook', u'VideoCard', u'Processor'])
        self.assertEquals([
            ('http://dummystore.com/?product=dm1-3060&price=199990',
             'Notebook',),
            ('http://dummystore.com/?product=sapphire-hd6850&'
             'availability=not-available&price=105500', 'VideoCard',),
            ('http://dummystore.com/?product='
             'intel-core-i5-2500k&price=145500', 'Processor',),
        ], product_urls_and_types)

    def test_product_urls_and_types_without_async(self):
        product_urls_and_types = self.store.product_urls_and_types(
            [u'Notebook', u'VideoCard', u'Processor'], async=False)
        self.assertEquals([
            ('http://dummystore.com/?product=dm1-3060&price=199990',
             'Notebook',),
            ('http://dummystore.com/?product=sapphire-hd6850&'
             'availability=not-available&price=105500', 'VideoCard',),
            ('http://dummystore.com/?product='
             'intel-core-i5-2500k&price=145500', 'Processor',),
        ], product_urls_and_types)

    def test_retrieve_product(self):
        product = self.store.retrieve_product(
            url=u'http://dummystore.com/?product=sapphire-hd6850&' \
                u'availability=not-available&price=105500',
            product_type=u'VideoCard')
        self.assertIsNone(product)

        product = self.store.retrieve_product(
            url=u'http://dummystore.com/?product=intel-core-i5-2500k&' \
                u'price=145500',
            product_type=u'Processor')
        expected_product = Product(u'intel-core-i5-2500k', Decimal(145500),
            u'http://dummystore.com/?product=intel-core-i5-2500k&price=145500',
            u'Processor', u'DummyStore')
        self.assertEquals(expected_product, product)

    def test_all_products_with_async(self):
        products = self.store.products([u'Notebook', u'VideoCard',
                                        u'Processor'])

        expected_products = {
            u'http://dummystore.com/?product=dm1-3060&price=199990':
                Product(u'dm1-3060', Decimal(199990),
                    u'http://dummystore.com/?product=dm1-3060&price=199990',
                    u'Notebook', u'DummyStore'),
            u'http://dummystore.com/?product=intel-core-i5-2500k&price=145500':
                Product(u'intel-core-i5-2500k', Decimal(145500),
                    u'http://dummystore.com/?product=intel-core-i5-2500k&'
                    u'price=145500', u'Processor', u'DummyStore'),
            u'http://dummystore.com/?product=sapphire-hd6850&availability='
            u'not-available&price=105500':
                None
        }

        self.assertEquals(expected_products, products)

    def test_subset_products_with_async(self):
        products = self.store.products(product_types=[u'Notebook'])
        expected_products = {
            u'http://dummystore.com/?product=dm1-3060&price=199990':
                Product(u'dm1-3060', Decimal(199990),
                    u'http://dummystore.com/?product=dm1-3060&price=199990',
                    u'Notebook', u'DummyStore')
        }

        self.assertEquals(expected_products, products)

    def test_all_products_without_async(self):
        products = self.store.products([u'Notebook', u'VideoCard',
                                        u'Processor'],
            async=False)
        expected_products = {
            u'http://dummystore.com/?product=dm1-3060&price=199990':
                Product(u'dm1-3060', Decimal(199990),
                    u'http://dummystore.com/?product=dm1-3060&price=199990',
                    u'Notebook', u'DummyStore'),
            u'http://dummystore.com/?product=intel-core-i5-2500k&price=145500':
                Product(u'intel-core-i5-2500k', Decimal(145500),
                    u'http://dummystore.com/?product=intel-core-i5-2500k&'
                    u'price=145500', u'Processor', u'DummyStore'),
            u'http://dummystore.com/?product=sapphire-hd6850&availability='
            u'not-available&price=105500':
                None
        }

        self.assertEquals(expected_products, products)

    def test_subset_products_without_async(self):
        products = self.store.products(product_types=[u'Notebook', ],
            async=False)
        expected_products = {
            u'http://dummystore.com/?product=dm1-3060&price=199990':
                Product(u'dm1-3060', Decimal(199990),
                    u'http://dummystore.com/?product=dm1-3060&price=199990',
                    u'Notebook', u'DummyStore')
        }

        self.assertEquals(expected_products, products)



from setuptools import setup

# We add the path to the tests so that the celeryconfig.py file may be found
import sys
sys.path.append('storescrapper/tests')

setup(
    name='storescrapper',
    version='0.1.5',
    author='Vijay Khemlani',
    author_email='vkhemlan@gmail.com',
    packages=[
        'storescrapper',
        'storescrapper.stores',
        'storescrapper.exceptions',
        'storescrapper.tests'
    ],
    scripts=[],
    url='https://github.com/SoloTodo/storescrapper',
    license='LICENSE.txt',
    description='Web scrapping API for selected stores',
    long_description=open('README.rst').read(),
    tests_require=[
        'nose>=1.1.2'
    ],
    test_suite='nose.collector',
    install_requires=[
        'mechanize>=0.2.5',
        'BeautifulSoup>=3.2.0',
        'kombu>=2.0.0',
        'celery>=2.4.6',
        ],
)



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Bip(Store):

    @classmethod
    def extract_links(cls, pageUrl):
        br = mechanize.Browser()
        data = br.open(pageUrl).get_data()
        soup = BeautifulSoup(data)
        links = soup.findAll('a', {'class': 'menuprod'})[::2]

        return links

    @classmethod
    def _retrieve_product(cls, url):
        br = mechanize.Browser()
        data = br.open(url).get_data()
        soup = BeautifulSoup(data)

        stock_info = soup.find('td', {'class': 'disp'})

        if not stock_info:
            return None

        stock_string = ''.join(str(stock) for stock in stock_info.contents)

        if 'Agotado' in stock_string:
            return None

        title_span = soup.find('td', {'class': 'menuprodg'})
        title = title_span.contents[0].strip()

        price_cell = soup.findAll('td', {'class': 'prcm'})
        price = Decimal(clean_price_string(price_cell[0].string))

        name = title.encode('ascii', 'ignore').strip()

        return [name, price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.bip.cl/ecommerce/'
        url_buscar_productos = 'index.php?modulo=busca&'

        url_extensions = [
            # Netbooks
            ['categoria=191', 'Notebook'],
            # Notebooks
            ['categoria=166', 'Notebook'],
            # Tarjetas de video
            ['categoria=118&categoria_papa=97', 'VideoCard'],
            # Proces Intel 775
            ['categoria=99&categoria_papa=111', 'Processor'],
            # Proces Intel 1155
            ['categoria=339&categoria_papa=111', 'Processor'],
            # Proces Intel 1156
            ['categoria=262&categoria_papa=111', 'Processor'],
            # Proces Intel 1366
            ['categoria=263&categoria_papa=111', 'Processor'],
            # Proces AMD AM2
            ['categoria=100&categoria_papa=111', 'Processor'],
            # Proces AMD AM3
            ['categoria=242&categoria_papa=111', 'Processor'],
            # LCD
            ['categoria=19', 'Monitor'],
            # Placas madre
            ['categoria=108', 'Motherboard'],
            # RAM PC
            ['categoria=132', 'Ram'],
            # RAM Notebook
            ['categoria=178', 'Ram'],
            # RAM Servidor
            ['categoria=179', 'Ram'],
            # Disco Duro 2,5'
            ['categoria=125&categoria_papa=123', 'StorageDrive'],
            # Disco Duro 3,5'
            ['categoria=124&categoria_papa=123', 'StorageDrive'],
            # Fuentes de poder
            ['categoria=88&categoria_papa=114', 'PowerSupply'],
        ]

        product_links = []

        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue

            page_number = 0

            while True:
                url_webpage = url_base + url_buscar_productos + url_extension \
                             + '&pagina=' + str(page_number)

                raw_links = cls.extract_links(url_webpage)
                if not raw_links:
                    break
                for raw_link in raw_links:
                    product_links.append([url_base + raw_link['href'], ptype])

                page_number += 1

        return product_links



from decimal import Decimal
from urllib2 import HTTPError
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


def unescape(s):
    s = s.replace('&lt;', '<')
    s = s.replace('&gt;', '>')
    s = s.replace('&quot;', '"')
    s = s.replace('&apos;', "'")
    s = s.replace('&amp;', '&')
    return s


class AbcDin(Store):

    @classmethod
    def _retrieve_product(cls, url):
        product_details_url = url.split('#')[1]
        try:
            product_webpage = mechanize.urlopen(product_details_url)
        except HTTPError:
            return None
        product_soup = BeautifulSoup(product_webpage.read())

        av_span = product_soup.find('div', {'id': 'caja-compra'}).find('span')
        if av_span:
            return None

        product_name = product_soup.find('td', {'id': 'mainDescr'})
        product_name = product_name.find('h2').contents[0]
        product_name = product_name.encode('ascii', 'ignore')

        try:
            product_price = product_soup.find('div', {'id': 'precioProducto'})
            product_price = product_price.find('strong').string
            product_price = product_price.split('Precio Tarjetas ABCDIN $')[1]
            product_price = Decimal(clean_price_string(product_price))
        except Exception:
            product_price = None

        if not product_price:
            try:
                price = product_soup.find('div', {'id': 'precioProducto'})
                price = price.find('strong').string
                price = price.replace('Precio Internet: $', '')
                product_price = Decimal(clean_price_string(price))
            except Exception:
                product_price = None

        if not product_price:
            price = product_soup.find('div', {'id': 'precioNormal'})
            price = price.find('span').string
            product_price = Decimal(clean_price_string(price))

        return [product_name, product_price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        cookies = mechanize.CookieJar()
        opener = mechanize.build_opener(mechanize.HTTPCookieProcessor(cookies))
        opener.addheaders = [('User-agent', 'Mozilla/5.0 (compatible)'),
            ('From', 'responsible.person@example.com')]
        mechanize.install_opener(opener)

        xml_resources = [
            ['notebooks', 'Notebook'],
            ['netbooks', 'Notebook'],
            ['LCD', 'Television'],
        ]

        product_links = []

        for xml_resource, ptype in xml_resources:
            if ptype not in product_types:
                continue
            # Obtain and parse HTML information of the base webpage
            base_url = 'https://www.abcdin.cl/abcdin/catabcdin.nsf/' \
                       '%28webProductosxAZ%29?readviewentries&' \
                       'restricttocategory='
            base_data = mechanize.urlopen(base_url + xml_resource)
            base_soup = BeautifulSoup(base_data.read())

            # Obtain the links to the other pages of the catalog (2, 3, ...)
            ntbks_data = base_soup.findAll('text')

            for ntbk_data in ntbks_data:
                ntbk_data = unescape(ntbk_data.contents[0])
                temp_soup = BeautifulSoup(ntbk_data)
                div = temp_soup.find('div')
                link = 'https://www.abcdin.cl/abcdin/abcdin.nsf#' \
                       'https://www.abcdin.cl' + div.find('a')['href']
                product_links.append([link, ptype])

        return product_links



from decimal import Decimal
from BeautifulSoup import BeautifulSoup
import mechanize
from storescrapper.store import Store
from storescrapper.utils import clean_price_string


class Clie(Store):

    @classmethod
    def _retrieve_product(cls, url):
        browser = mechanize.Browser()
        product_data = browser.open(url).get_data()
        product_soup = BeautifulSoup(product_data)

        availability_text = product_soup.findAll('td',
                {'class': 'texto-neg-bold'})[-1].find('a').string.strip()
        if availability_text[0] == '0':
            return None

        product_name = product_soup.find('td', {'class': 'tit-nar-bold'})
        product_name = product_name.contents[0].split('&#8226;')[0]
        product_name = product_name.replace('&nbsp;&raquo; ', '').strip()

        product_price = product_soup.find('td',
                {'background': 'images/ficha/bg_efectivo_d.gif'})
        product_price = Decimal(clean_price_string(
            product_price.find('a').string))

        if not product_price:
            return None

        return [product_name, product_price]

    @classmethod
    def _product_urls_and_types(cls, product_types):
        url_base = 'http://www.clie.cl/'
        browser = mechanize.Browser()

        url_extensions = [
            ['561', 'Notebook'],
            ['542', 'Notebook'],
            ['580', 'Notebook'],
            ['564', 'Notebook'],
            ['581', 'Notebook'],
            ['562', 'Notebook'],
            ['579', 'Notebook'],
            ['575', 'Notebook'],
            ['612', 'Notebook'],
            ['598', 'Notebook'],
            ['596', 'Notebook'],
            ['595', 'Notebook'],
            ['178', 'Notebook'],
            ['500', 'Notebook'],
            ['158', 'Notebook'],
            ['307', 'Notebook'],
            ['308', 'Notebook'],
            ['646', 'Processor'],
            ['167', 'Monitor'],
            ['551', 'Monitor'],
            ['19', 'Monitor'],
            ['536', 'Monitor'],
            ['310', 'Television'],
            ['266', 'Television'],
            ['550', 'Television'],
            ['560', 'Television'],
            ['632', 'Monitor'],
            ['616', 'Monitor'],
            ['614', 'Television'],
            ['614', 'Television'],
            ['26', 'Ram'],
            ['446', 'Ram'],
            ['438', 'StorageDrive'],
            ['434', 'StorageDrive'],
            ['738', 'StorageDrive'],
            ['642', 'StorageDrive'],
        ]

        product_links = []

        for url_extension, ptype in url_extensions:
            if ptype not in product_types:
                continue
            num_page = 1
            while True:
                url_webpage = url_base + '?ver=4&categoria_producto=' + \
                              url_extension + '&pagina=' + str(num_page)

                base_soup = BeautifulSoup(browser.open(url_webpage).get_data())
                product_cells = base_soup.findAll('td', {'colspan': '2'})[1:]

                if not product_cells:
                    break

                for product_cell in product_cells:
                    link = product_cell.find('a')
                    url = link['onclick'].split('\'')[1]
                    product_links.append([url_base + url, ptype])

                num_page += 1

        return product_links
